/*
    Copyright (  C ) 2003 Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation;
    version 2 of the License.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.

*/

#include "klevelmeter_linebars.h"

#include <kdebug.h>
#include <qpainter.h>

KLevelMeter_LineBars::KLevelMeter_LineBars( Arts::KLevelMeter_impl* impl, QWidget* parent, long substyle, long count, float _dbmin, float _dbmax ) : KLevelMeter_Template( impl, parent, substyle, count, _dbmin, _dbmax ) {
kdDebug()<<"KLevelMeter_LineBars::KLevelMeter_LineBars( Arts::KLevelMeter_impl* "<<impl<<", QWidget* "<<parent<<", long "<<substyle<<", long "<<count<<", float "<<_dbmin<<", float "<<_dbmax<<" )"<<endl;
	this->setMinimumSize( 5, 5 );
}

void KLevelMeter_LineBars::invalue( float n, float p ) {
//kdDebug()<<"KLevelMeter_LineBars::invalue( float n )"<<endl;
	_value = dbtovol( amptodb( n ) );
	_peak  = dbtovol( amptodb( p ) );
	repaint();
}

void KLevelMeter_LineBars::volume( float n ) {
	_volume = dbtondb( amptodb( n ) );
	repaint();
}

void KLevelMeter_LineBars::paintEvent( QPaintEvent* ) {
	QPainter p( this );
	p.setBrush( ( _peak<1 )?color( _value ):QColor( 255,0,0 ) );
	p.setPen( NoPen );
	p.translate( 0, rect().bottom() );
	// Value
	int h = int( this->height()*_value );
	p.drawRect( 0, 0, this->width(), -h );
	// PeakBar
	if ( _peak > 1.0/1000 ) {
		p.setPen( QColor( 0,0,255 ) );
		h = int( this->height()*_peak );
		p.drawLine( 0, -h, this->width(), -h );
	}
	// Volume
	if ( _volume > 1.0/1000 ) {
		p.setPen( QPen( QColor( 0,0,0 ), 2 ) );
		h = int( this->height()*_volume );
		p.drawLine( 0, -h, this->width(), -h );
	}
}

/**
	Planned feature: a little Tooltip showing the actual value of the volume in deziBel and perhaps as linear scaleFactor
*/
void KLevelMeter_LineBars::mouseMoveEvent( QMouseEvent* qme ) {
//kdDebug()<<"KLevelMeter_LineBars::mouseMoveEvent(QMouseEvent* "<<qme<<" )"<<endl;
	float db = ndbtodb( float( this->height()-qme->y() )/this->height() );
//kdDebug()<<"qme.y()="<<this->height()-qme->y()<<" db="<<db<<" dbtoamp(db)="<<dbtoamp( db )<<endl;
	_impl->__internal_volumevalue( dbtoamp( db ) );
}

#include <klevelmeter_linebars.moc>
// vim: sw=4 ts=4
