/*

    Copyright (C) 2003 Nikolas Zimmermann <wildfox@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.

*/

#include <stdlib.h>

#include "globals.h"
#include "rawreader.h"

RawReader::RawReader() : ReaderWriter()
{
}

RawReader::~RawReader()
{
}

Q_INT32 RawReader::readFile(Q_INT32 *buf, Q_INT32 nsamp)
{
	Q_INT32 len, done = 0;

	len = QMIN(nsamp, (Q_INT32) (m_count - m_pos) / m_signalSize);

	if(len)
	{
		readBuffer(buf + done, m_buf + m_pos, len);
		m_pos += (len * m_signalSize);
		done += len;		
	}
	
	int i = 0;
	while(done < nsamp)
	{
		if(!m_eof && (m_count == 0 ||
					  m_pos >= (m_count - m_signalSize + 1)))
		{
			for(i = 0; i < (Q_INT32) (m_count - m_pos); i++)
				m_buf[i] = m_buf[m_pos + i];

			i = (m_count - m_pos);
			m_pos = 0;

			m_count = fread(m_buf + i, 1, m_size - i, openStream());

			if(m_count == 0)
				m_eof = true;

			m_count += i;
		}

		len = QMIN((nsamp - done), (Q_INT32) (m_count - m_pos) / m_signalSize);

		if(len)
		{
			readBuffer(buf + done, m_buf + m_pos, len);
			m_pos += (len * m_signalSize);
			done += len;
		}

		if(m_eof)
			break;
	}

	return done;
}

void RawReader::readBuffer(Q_INT32 *buf1, char *buf2, Q_INT32 len)
{
    while(len)
    {
		Q_UINT16 datum;

		datum = *((Q_UINT16 *) buf2);
		buf2++; buf2++;

		if(GLOBAL_SWAP)
			datum =	((datum >> 8) | (datum << 8)) & 0xffff;
		
		*buf1++ = ((Q_INT32)(datum) << 16);
		len--;
    }
}
